
%define build_all 1
%define build_arm %{build_all}
%define build_aarch64 %{build_all}

Summary: A GNU collection of cross-compilation binary utilities
Name: fedora-linaro-cross-binutils
Version: 2.23.51.20130111
Release: 1%{?dist}
License: GPLv3+
Group: Development/Tools
URL: http://sources.redhat.com/binutils

Source: binutils-%{version}.tar.gz
Source2: binutils-2.19.50.0.1-output-format.sed

Patch01: binutils-2.20.51.0.2-libtool-lib64.patch
Patch02: binutils-2.20.51.0.10-ppc64-pie.patch
Patch03: binutils-2.20.51.0.2-ia64-lib64.patch
Patch04: binutils-2.20.51.0.2-version.patch
Patch05: binutils-2.20.51.0.2-set-long-long.patch
Patch06: binutils-2.20.51.0.10-copy-osabi.patch
Patch07: binutils-2.20.51.0.10-sec-merge-emit.patch
# Enable -zrelro by default: BZ #621983
Patch08: binutils-2.22.52.0.1-relro-on-by-default.patch
# Local patch - export demangle.h with the binutils-devel rpm.
Patch09: binutils-2.22.52.0.1-export-demangle.h.patch
# Disable checks that config.h has been included before system headers.  BZ #845084
Patch10: binutils-2.22.52.0.4-no-config-h-check.patch

Buildroot: %(mktemp -ud %{_tmppath}/%{name}-%{version}-%{release}-XXXXXX)
BuildRequires: texinfo >= 4.0, gettext, flex, bison, zlib-devel
# Required for: ld-bootstrap/bootstrap.exp bootstrap with --static
# It should not be required for: ld-elf/elf.exp static {preinit,init,fini} array
Conflicts: gcc-c++ < 4.0.0
Provides: bundled(libiberty)

%description
Binutils is a collection of binary utilities, including ar (for
creating, modifying and extracting from archives), as (a family of GNU
assemblers), gprof (for displaying call graph profile data), ld (the
GNU linker), nm (for listing symbols from object files), objcopy (for
copying and translating object files), objdump (for displaying
information from object files), ranlib (for generating an index for
the contents of an archive), readelf (for displaying detailed
information about binary files), size (for listing the section sizes
of an object or archive file), strings (for listing printable strings
from files), strip (for discarding symbols), and addr2line (for
converting addresses to file and line).

These are the Linaro versions of these cross-tools and are to be
used to generate object files for a standard Fedora development
environment, NOT for Linaro.

%package -n fedora-linaro-cross-binutils-common
Summary: Cross-build binary utility documentation and translation files
Group: Development/Tools
BuildArch: noarch
%description -n fedora-linaro-cross-binutils-common
Documentation, manual pages and translation files for cross-build binary image
generation, manipulation and query tools.

These are the Linaro versions of these cross-tools and are to be
used to generate object files for a standard Fedora development
environment, NOT for Linaro.

%define do_package() \
%if %2 \
%package -n fedora-linaro-binutils-%1 \
Summary: Cross-build binary utilities for %1 \
Group: Development/Tools \
Requires: fedora-linaro-cross-binutils-common == %{version}-%{release} \
%description -n fedora-linaro-binutils-%1 \
Cross-build binary image generation, manipulation and query tools. \
 \
These are the Linaro versions of these cross-tools and are to be \
used to generate object files for a standard Fedora development \
environment, NOT for Linaro. \
%endif

%do_package arm-linux-gnu	%{build_arm}
%do_package aarch64-linux-gnu	%{build_aarch64}

# Where the binaries aimed at gcc will live (ie. /usr/<target>/bin/)
%define auxbin_prefix %{_exec_prefix}/fedora-linaro-cross

###############################################################################
#
# Preparation
#
###############################################################################
%prep

%define srcdir binutils-%{version}
%setup -q -n %{srcdir} -c
cd %{srcdir}
%patch01 -p0 -b .libtool-lib64~
%patch02 -p0 -b .ppc64-pie~
%ifarch ia64
%if "%{_lib}" == "lib64"
%patch03 -p0 -b .ia64-lib64~
%endif
%endif
%patch04 -p0 -b .version~
%patch05 -p0 -b .set-long-long~
%patch06 -p0 -b .copy-osabi~
%patch07 -p0 -b .sec-merge-emit~
%if 0%{?fedora} >= 18 || 0%{?rhel} >= 7
%patch08 -p0 -b .relro~
%endif
%patch09 -p0 -b .export-demangle-h~
%patch10 -p0 -b .no-config-h-check~

# We cannot run autotools as there is an exact requirement of autoconf-2.59.

# $PACKAGE is used for the gettext catalog name.
sed -i -e 's/^ PACKAGE=/ PACKAGE=fedora-linaro-cross-/' */configure
# Undo the name change to run the testsuite.
for tool in binutils gas ld
do
  sed -i -e "2aDEJATOOL = $tool" $tool/Makefile.am
  sed -i -e "s/^DEJATOOL = .*/DEJATOOL = $tool/" $tool/Makefile.in
done
touch */configure

function prep_target () {
    target=$1
    cond=$2

    if [ $cond = 1 ]
    then
	echo $1 >&5
    fi
}

cd ..
(
    prep_target arm-linux-gnu		%{build_arm}
    prep_target aarch64-linux-gnu	%{build_aarch64}
) 5>target.list

n=0
for target in `cat target.list`
do
    n=1
    break
done
if [ $n = 0 ]
then
    echo "No targets selected" >&2
    exit 8
fi

###############################################################################
#
# Build
#
###############################################################################
%build

function config_target () {
    arch=$1
    prefix=fedora-linaro-$arch-
    build_dir=$arch

    case $arch in
	arm-*)		target=arm-linux-gnueabi;;
	aarch64-*)	target=aarch64-linux-gnu;;
	*)		target=$arch;;
    esac

    echo $arch: target is $target
    export CFLAGS="$RPM_OPT_FLAGS"
    CARGS=

    case $target in i?86*|sparc*|ppc*|s390*|sh*|aarch64*)
	    CARGS="$CARGS --enable-64-bit-bfd"
	    ;;
    esac

    mkdir $build_dir
    cd $build_dir

    # We could optimize the cross build size by --enable-shared but the produced
    # binaries may be less convenient in the embedded environment.
    LDFLAGS='-Wl,-z,relro ' \
    ../%{srcdir}/configure \
	--disable-dependency-tracking \
	--disable-silent-rules \
	--enable-checking \
	--prefix=%{_prefix} \
	--exec-prefix=%{auxbin_prefix} \
	--bindir=%{_bindir} \
	--sbindir=%{_sbindir} \
	--sysconfdir=%{_sysconfdir} \
	--datadir=%{_datadir} \
	--includedir=%{_includedir} \
	--libdir=%{_libdir} \
	--libexecdir=%{_libexecdir} \
	--localstatedir=%{_localstatedir} \
	--sharedstatedir=%{_sharedstatedir} \
	--mandir=%{_mandir} \
	--infodir=%{_infodir} \
	--build=%{_target_platform} \
	--host=%{_target_platform} \
	--target=$target \
	--program-prefix=$prefix \
	--disable-shared \
	--disable-install_libbfd \
	$CARGS
    cd ..
}

for target in `cat target.list`
do
    config_target $target
done

function build_target () {
    arch=$1
    make -C $arch %{_smp_mflags} tooldir=%{_prefix} all
}

for target in `cat target.list`
do
    build_target $target
done

# for documentation purposes only
mkdir fedora-linaro-cross-binutils
cd fedora-linaro-cross-binutils
../%{srcdir}/configure \
    --disable-dependency-tracking \
    --disable-silent-rules \
    --prefix=%{_prefix} \
    --exec-prefix=%{auxbin_prefix} \
    --bindir=%{_bindir} \
    --sbindir=%{_sbindir} \
    --sysconfdir=%{_sysconfdir} \
    --datadir=%{_datadir} \
    --includedir=%{_includedir} \
    --libdir=%{_libdir} \
    --libexecdir=%{_libexecdir} \
    --localstatedir=%{_localstatedir} \
    --sharedstatedir=%{_sharedstatedir} \
    --mandir=%{_mandir} \
    --infodir=%{_infodir} \
    --program-prefix=fedora-linaro-cross- \
    --disable-shared
make %{_smp_mflags} tooldir=%{_prefix} all
cd ..

###############################################################################
#
# Installation
#
###############################################################################
%install
rm -rf %{buildroot}

function install_bin () {
    cross=$1
    make install -C $cross DESTDIR=%{buildroot}
}

for target in `cat target.list`
do
    echo "=== INSTALL target $target ==="
    install_bin $target
done

echo "=== INSTALL man targets ==="
make install-man1 -C fedora-linaro-cross-binutils/binutils/doc DESTDIR=%{buildroot}
make install-man1 -C fedora-linaro-cross-binutils/gas/doc DESTDIR=%{buildroot}
make install-man1 -C fedora-linaro-cross-binutils/ld DESTDIR=%{buildroot}
make install-man1 -C fedora-linaro-cross-binutils/gprof DESTDIR=%{buildroot}

echo "=== INSTALL po targets ==="
make install -C fedora-linaro-cross-binutils/binutils/po DESTDIR=%{buildroot}
make install -C fedora-linaro-cross-binutils/gas/po DESTDIR=%{buildroot}
make install -C fedora-linaro-cross-binutils/ld/po DESTDIR=%{buildroot}
make install -C fedora-linaro-cross-binutils/gprof/po DESTDIR=%{buildroot}
make install -C fedora-linaro-cross-binutils/bfd/po DESTDIR=%{buildroot}
make install -C fedora-linaro-cross-binutils/opcodes/po DESTDIR=%{buildroot}

# For fedora-linaro-cross-binutils we drop the documentation.
echo "=== REMOVE documentation ==="
rm -rf %{buildroot}%{_infodir}
rm -f %{buildroot}%{_infodir}/dir

echo "=== REMOVE libraries and scripts ==="
rm -rf %{buildroot}%{_libdir}/libiberty.a
rm -rf %{buildroot}%{auxbin_prefix}/*/lib/ldscripts
rmdir %{buildroot}%{auxbin_prefix}/*/lib || :

echo "=== BUILD file lists ==="
function build_file_list () {
    arch=$1
    cpu=${arch%%%%-*}

    case $cpu in
	*)		target_cpu=$cpu;;
    esac

    (
	echo '%%defattr(-,root,root,-)'
	echo %{_bindir}/fedora-linaro-$arch-[!l]\*
	echo %{_bindir}/fedora-linaro-$arch-ld\*
	echo %{auxbin_prefix}/$target_cpu-*/bin/\*
	echo %{_mandir}/man1/fedora-linaro-$arch-\*
    ) >files.$arch
}

for target in `cat target.list`
do
    build_file_list $target
done

# All the installed manual pages and translation files for each program are the
# same, so symlink them to the core package
echo "=== CROSSLINK man pages ==="
cd %{buildroot}%{_mandir}/man1
for i in fedora-linaro-cross-*.1*
do
    j=${i#fedora-linaro-cross-}

    for k in *-$j
    do
	if [ $k != $i ]
	then
	    ln -sf $i $k
	fi
    done
done


# Add ld.bfd manual pages
find * -name "*ld.1*" -a ! -name "fedora-linaro-cross-ld.1*" -print |
while read x
do
    y=`echo $x | sed -e s/ld[.]1/ld.bfd.1/`
    ln -s $x $y
done

cd -

# Find the language files which only exist in the common package
(
    %find_lang fedora-linaro-cross-binutils
    %find_lang fedora-linaro-cross-opcodes
    %find_lang fedora-linaro-cross-bfd
    %find_lang fedora-linaro-cross-gas
    %find_lang fedora-linaro-cross-ld
    %find_lang fedora-linaro-cross-gprof
    cat fedora-linaro-cross-binutils.lang
    cat fedora-linaro-cross-opcodes.lang
    cat fedora-linaro-cross-bfd.lang
    cat fedora-linaro-cross-gas.lang
    cat fedora-linaro-cross-ld.lang
    cat fedora-linaro-cross-gprof.lang
) >files.cross



###############################################################################
#
# Cleanup
#
###############################################################################
%clean
rm -rf %{buildroot}

###############################################################################
#
# Filesets
#
###############################################################################
%files -n fedora-linaro-cross-binutils-common -f files.cross
%doc %{srcdir}/README
%doc %{srcdir}/COPYING*
%{_mandir}/man1/fedora-linaro-cross-*

%define do_files() \
%if %2 \
%files -n fedora-linaro-binutils-%1 -f files.%1 \
%endif

%do_files arm-linux-gnu		%{build_arm}
%do_files aarch64-linux-gnu	%{build_aarch64}

%changelog
* Fri Jan 11 2013 Al Stone <ahs3@redhat.com> - 2.23.51.20130111
- Clone from cross-binutils to provide cross-tools built from Linaro
  sources and targeted for Linaro Debian-based systems

